@@ -163,7 +163,7 @@ class AsyncComm(ProxyObject, ComponentBaseUnregistered):
         if self._backlog_thread is not None:
             self.backlog_thread.set_break_flag()
         self.backlog_ready.set()
-        if wait and (self._backlog_thread is not None):
+        if wait and (self._backlog_thread is not None):  # pragma: intermittent
             self.backlog_thread.wait(key=str(uuid.uuid4()))
         if hasattr(self._wrapped, '_close_backlog'):
             self._wrapped._close_backlog(wait=wait)
@@ -515,15 +515,15 @@ class AsyncComm(ProxyObject, ComponentBaseUnregistered):
         if self.n_msg_backlog == 0:
             self.verbose_debug("No messages waiting.")
             if self.is_closed:
-                self.info(("No messages waiting and comm closed."
-                           "%s, %s, %s")
-                          % (self.backlog_thread is not None,
-                             not self.backlog_thread.was_break,
-                             self.backlog_thread.is_alive()))
-                self.printStatus()
+                self.debug(("No messages waiting and comm closed."
+                            "%s, %s, %s")
+                           % (self.backlog_thread is not None,
+                              not self.backlog_thread.was_break,
+                              self.backlog_thread.is_alive()))
+                self.printStatus(level='debug')
                 if self.backlog_thread.was_break:
-                    self.info("Break stack:\n%s",
-                              self.backlog_thread.break_stack)
+                    self.debug("Break stack:\n%s",
+                               self.backlog_thread.break_stack)
                 out = CommBase.CommMessage(flag=CommBase.FLAG_FAILURE)
             else:
                 out = CommBase.CommMessage(flag=CommBase.FLAG_EMPTY,
