@@ -102,12 +102,11 @@ def example_decorator(name, x, iter_over, timeout):
     """
 
     def deco(func):
-        deco_list = [
-            timeout_dec(timeout=timeout),
-            unittest.skipIf(
-                (not tools.check_environ_bool('YGG_ENABLE_EXAMPLE_TESTS')),
-                "Example tests not enabled."),
-        ]
+        add_timeout_dec = True
+        flag = (not tools.check_environ_bool('YGG_ENABLE_EXAMPLE_TESTS'))
+        if not flag:
+            add_timeout_dec = False
+        deco_list = [unittest.skipIf(flag, "Example tests not enabled.")]
         for i, k in enumerate(iter_over):
             v = x[i]
             flag = None
@@ -120,10 +119,22 @@ def example_decorator(name, x, iter_over, timeout):
                     if not tools.is_lang_installed(vv):
                         flag = False
                         break
+                    else:
+                        try:
+                            check_enabled_languages(vv)
+                        except unittest.SkipTest as e:
+                            msg = str(e)
+                            flag = False
+                            break
             if flag is not None:
+                if not flag:
+                    # Don't add timeout if the test is going to be skipped
+                    add_timeout_dec = False
                 if msg is None:
                     msg = "%s %s not installed." % (k.title(), v)
                 deco_list.append(unittest.skipIf(not flag, msg))
+        if add_timeout_dec:
+            deco_list.insert(0, timeout_dec(timeout=timeout))
         for v in deco_list:
             func = v(func)
         return func
@@ -334,6 +345,7 @@ class ExampleTstBase(YggTestBase, tools.YggClass):
         self.runner = runner.get_runner(self.yaml, namespace=self.namespace,
                                         production_run=True)
         self.runner.run()
+        self.runner.printStatus()
         if self.expects_error:
             assert(self.runner.error_flag)
         else:
