@@ -1,11 +1,11 @@
-from yggdrasil import languages, tools, platform
+from yggdrasil import languages, constants, platform
 from yggdrasil.tests import assert_raises
 
 
 def test_get_language_dir():
     r"""Test the get_language_dir method."""
     assert_raises(ValueError, languages.get_language_dir, 'invalid')
-    test_lang = tools.get_supported_lang()
+    test_lang = constants.LANGUAGES_WITH_ALIASES['all']
     test_skip = ['make', 'cmake', 'executable', 'timesync', 'osr']
     for lang in test_lang:
         if lang in test_skip:
@@ -15,9 +15,10 @@ def test_get_language_dir():
 
 def test_get_language_ext():
     r"""Test the get_language_ext method."""
-    test_lang = tools.get_supported_lang()
+    test_lang = constants.LANGUAGES_WITH_ALIASES['all']
     for lang in test_lang:
-        if (lang == 'executable') and (not platform._is_win):
+        if ((((lang == 'executable') and (not platform._is_win))
+             or (lang in ['function']))):
             assert_raises(ValueError, languages.get_language_ext, lang)
             languages.get_language_ext(lang, default='')
         else:
