@@ -214,6 +214,104 @@ def get_model_form_schema(fname_dst=None):
     return out
 
 
+def update_constants():
+    r"""Update constants.py with info from the schema."""
+    from yggdrasil.components import import_component
+    from yggdrasil.drivers.CompiledModelDriver import (
+        get_compilation_tool_registry)
+
+    def as_lines(x, newline='\n', key_order=None):
+        out = ""
+        if isinstance(x, dict):
+            if key_order is None:
+                key_order = list(sorted(x.keys()))
+            out += "{" + newline
+            for k in key_order:
+                v = x[k]
+                out += "    %s: %s," % (
+                    repr(k), as_lines(v, newline=(newline + '    '))) + newline
+            out += "}"
+        elif isinstance(x, list):
+            out += "[" + newline
+            out += "    " + ", ".join(
+                [as_lines(xx, newline=(newline + '    ')) for xx in x]) + "]"
+        else:
+            out += repr(x)
+        return out
+
+    separator = (
+        "\n# ======================================================\n"
+        "# Do not edit this file past this point as the following\n"
+        "# is generated by yggdrasil.schema.update_constants\n"
+        "# ======================================================\n")
+    filename = os.path.join(os.path.dirname(__file__), 'constants.py')
+    s = get_schema()
+    drivers = {k: import_component('model', k) for k in s['model'].subtypes}
+    language_cat = ['compiled', 'interpreted', 'build', 'dsl', 'other']
+    typemap = {'compiler': 'compiled', 'interpreter': 'interpreted'}
+    lang2ext = {'yaml': '.yml', 'executable': '.exe'}
+    languages = {k: [] for k in language_cat}
+    languages_with_aliases = {k: [] for k in language_cat}
+    compiler_env_vars = {}
+    compilation_tool_vars = {}
+    complete = []
+    for k, drv in drivers.items():
+        if drv.language in complete:
+            continue
+        complete.append(drv.language)
+        drv_type = typemap.get(drv.executable_type, drv.executable_type)
+        if drv.language_ext:
+            if k not in lang2ext:
+                assert(isinstance(drv.language_ext, list))
+                lang2ext[k] = drv.language_ext[0]
+            for ka in drv.language_aliases:
+                lang2ext[ka] = lang2ext[k]
+        languages.setdefault(drv_type, [])
+        languages[drv_type].append(drv.language)
+        languages_with_aliases.setdefault(drv_type, [])
+        languages_with_aliases[drv_type].append(drv.language)
+        languages_with_aliases[drv_type] += drv.language_aliases
+    for x in ['compiler', 'linker', 'archiver']:
+        reg = get_compilation_tool_registry(x).get('by_language', {})
+        for lang, tools in reg.items():
+            for v in tools.values():
+                k = v.toolname
+                if v.is_build_tool or (k in compilation_tool_vars):
+                    continue
+                compilation_tool_vars[k] = {
+                    'exec': v.default_executable_env,
+                    'flags': v.default_flags_env}
+                if (x == 'compiler') and (lang not in compiler_env_vars):
+                    compiler_env_vars[lang] = compilation_tool_vars[k].copy()
+    language_cat = list(languages.keys())
+    with open(filename, 'r') as fd:
+        lines = [fd.read().split(separator)[0], separator[1:]]
+    lines += [
+        "LANG2EXT = %s" % as_lines(lang2ext),
+        "EXT2LANG = {v: k for k, v in LANG2EXT.items()}",
+        "LANGUAGES = %s" % as_lines(languages, key_order=language_cat)]
+    lines.append(
+        "LANGUAGES['all'] = (\n    LANGUAGES[%s]"
+        % repr(language_cat[0]))
+    lines += ["    + LANGUAGES[%s]" % repr(k)
+              for k in language_cat[1:]]
+    lines[-1] += ")"
+    lines += [
+        "LANGUAGES_WITH_ALIASES = %s" % as_lines(languages_with_aliases,
+                                                 key_order=language_cat)]
+    lines.append(
+        "LANGUAGES_WITH_ALIASES['all'] = (\n    LANGUAGES_WITH_ALIASES[%s]"
+        % repr(language_cat[0]))
+    lines += ["    + LANGUAGES_WITH_ALIASES[%s]" % repr(k)
+              for k in language_cat[1:]]
+    lines[-1] += ")"
+    lines += [
+        "COMPILER_ENV_VARS = %s" % as_lines(compiler_env_vars),
+        "COMPILATION_TOOL_VARS = %s" % as_lines(compilation_tool_vars)]
+    with open(filename, 'w') as fd:
+        fd.write('\n'.join(lines) + '\n')
+
+
 class ComponentSchema(object):
     r"""Schema information for one component.
 
@@ -417,7 +515,7 @@ class ComponentSchema(object):
         subt_schema = schema['allOf'][1]['anyOf']
         # Determine subtype key
         subt_overlap = set(list(subt_schema[0]['properties'].keys()))
-        subt_props = copy.deepcopy(subt_overlap)
+        subt_props = subt_overlap.copy()
         for v in subt_schema[1:]:
             ikeys = set(list(v['properties'].keys()))
             subt_props |= ikeys
@@ -463,7 +561,7 @@ class ComponentSchema(object):
         for x in schema_classes.values():
             if out is None:
                 out = cls(schema_type, x._schema_subtype_key, **kwargs)
-            out.append(x)
+            out.append(x, verify=True)
         return out
 
     @property
@@ -526,16 +624,19 @@ class ComponentSchema(object):
         r"""list: All available classes for this schema."""
         return sorted([k for k in self.schema_subtypes.keys()])
 
-    def append(self, comp_cls):
+    def append(self, comp_cls, verify=False):
         r"""Append component class to the schema.
 
         Args:
             comp_cls (class): Component class that should be added.
+            verify (bool, optional): If True, verify the schema after
+                adding the component class. Defaults to False.
 
         """
         assert(comp_cls._schema_type == self.schema_type)
         assert(comp_cls._schema_subtype_key == self.subtype_key)
         name = comp_cls.__name__
+        # name = '%s.%s' % (comp_cls.__module__, comp_cls.__name__)
         # Append subtype
         subtype_list = getattr(comp_cls, '_%s' % self.subtype_key, None)
         if not isinstance(subtype_list, list):
@@ -604,6 +705,8 @@ class ComponentSchema(object):
                 if k != self.subtype_key:
                     old.pop('description', None)
                     new.pop('description', None)
+                    old.pop('default', None)
+                    new.pop('default', None)
                     if old != new:  # pragma: debug
                         raise ValueError(
                             ("Schema for property '%s' of class '%s' "
@@ -620,7 +723,8 @@ class ComponentSchema(object):
             self._base_schema['properties'] = new_base_prop
         self._storage[name] = copy.deepcopy(new_schema)
         # Verify that the schema is valid
-        metaschema.validate_schema(self.schema)
+        if verify:
+            metaschema.validate_schema(self.schema)
 
 
 class SchemaRegistry(object):
@@ -655,19 +759,21 @@ class SchemaRegistry(object):
             # Create schemas for each component
             for k, v in registry.items():
                 icomp = ComponentSchema.from_registry(k, v, schema_registry=self)
-                self.add(k, icomp)
+                self.add(k, icomp, verify=True)
 
-    def add(self, k, v):
+    def add(self, k, v, verify=False):
         r"""Add a new component schema to the registry."""
         self._cache = {}
         self._storage[k] = v
-        metaschema.validate_schema(self.schema)
+        if verify:
+            metaschema.validate_schema(self.schema)
 
     def get(self, k, *args, **kwargs):
         r"""Return a component schema from the registry."""
         return self._storage.get(k, *args, **kwargs)
 
-    def get_definitions(self, relaxed=False, allow_instance=False, for_form=False):
+    def get_definitions(self, relaxed=False, allow_instance=False,
+                        for_form=False, dont_copy=False):
         r"""Get schema definitions for the registered components.
 
         Args:
@@ -681,6 +787,8 @@ class SchemaRegistry(object):
             for_form (bool, optional): If True, the returned schema will be
                 formatted for easy parsing by form generation tools. Defaults
                 to False. Causes relaxed and allow_instance to be ignored.
+            dont_copy (bool, optional): If True, a the cached definitions
+                are returned without copying. Defaults to False.
 
         Returns:
             dict: Schema defintiions for each of the registered components.
@@ -702,7 +810,10 @@ class SchemaRegistry(object):
             for k in self.required_components:
                 out.setdefault(k, {'type': 'string'})
             self._cache[cache_key] = out
-        return copy.deepcopy(self._cache[cache_key])
+        out = self._cache[cache_key]
+        if not dont_copy:
+            out = copy.deepcopy(out)
+        return out
 
     def get_schema(self, relaxed=False, allow_instance=False, for_form=False):
         r"""Get the schema defining this component.
@@ -738,7 +849,7 @@ class SchemaRegistry(object):
                    'type': 'object',
                    'definitions': self.get_definitions(
                        relaxed=relaxed, allow_instance=allow_instance,
-                       for_form=for_form),
+                       for_form=for_form, dont_copy=True),
                    'required': ['models'],
                    'additionalProperties': False,
                    'properties': SchemaDict(
@@ -1250,24 +1361,6 @@ def standardize(instance, keys, is_singular=False, suffixes=None, altkeys=None):
                 instance[k][i] = {'name': instance[k][i]}
 
 
-def normalize_function_file(cond, working_dir):
-    r"""Normalize functions which use relative paths.
-
-    Args:
-        cond (str): Function expression.
-        working_dir (str): Full path to working directory that
-            should be used to normalized the function path.
-
-    Returns:
-        str: Normalized function expression.
-
-    """
-    mod_file, func_name = cond.split(':', 1)
-    if mod_file.endswith('.py') and (not os.path.isabs(mod_file)):
-        mod_file = os.path.normpath(os.path.join(working_dir, mod_file))
-    return ':'.join([mod_file, func_name])
-
-
 @SchemaRegistry.register_normalizer(tuple())
 def _normalize_root(normalizer, value, instance, schema):
     r"""Decorate normalizer."""
@@ -1289,6 +1382,8 @@ def _normalize_modelio_first(normalizer, value, instance, schema):
     iodict = getattr(normalizer, 'iodict', None)
     if isinstance(instance, dict):
         standardize(instance, ['inputs', 'outputs'])
+        if instance.get('language', None) == 'timesync':
+            instance.setdefault('args', [])
         prefix = '%s:' % instance['name']
         for io in ['inputs', 'outputs']:
             if len(instance[io]) == 0:
@@ -1304,13 +1399,10 @@ def _normalize_modelio_first(normalizer, value, instance, schema):
                 if not x.get('is_default', False):
                     x.setdefault('working_dir', instance['working_dir'])
                 if 'default_file' in x:
+                    if isinstance(x['default_file'], str):
+                        x['default_file'] = {'name': x['default_file']}
                     x['default_file'].setdefault('working_dir',
                                                  instance['working_dir'])
-                for k in ['filter', 'transform']:
-                    x_k = x.get(k, None)
-                    if isinstance(x_k, dict) and ('function' in x_k):
-                        x[k]['function'] = normalize_function_file(
-                            x[k]['function'], instance['working_dir'])
     return instance
 
 
@@ -1415,14 +1507,6 @@ def _normalize_connio_first(normalizer, value, instance, schema):
     if isinstance(instance, dict):
         standardize(instance, ['inputs', 'outputs'], suffixes=['_file', '_files'],
                     altkeys=[['from', 'to']])
-        if instance.get('working_dir', False):
-            for io in ['inputs', 'outputs']:
-                for x in instance[io]:
-                    for k in ['filter', 'transform']:
-                        x_k = x.get(k, None)
-                        if isinstance(x_k, dict) and ('function' in x_k):
-                            x[k]['function'] = normalize_function_file(
-                                x[k]['function'], instance['working_dir'])
         # Handle indexed inputs/outputs
         for io in ['inputs', 'outputs']:
             pruned = []
