@@ -2,6 +2,7 @@ import os
 import sys
 import copy
 import unittest
+import flaky
 from yggdrasil import tools, timing, platform
 from yggdrasil.tests import YggTestClass, assert_raises, long_running
 
@@ -12,7 +13,7 @@ _test_nrep = 1
 _test_lang = 'c'
 # On windows, it's possible to not have a C/C++ communication library installed
 if 'c' not in timing.get_lang_list():  # pragma: windows
-    _test_lang = 'python'
+    _test_lang = 'python'  # pragma: testing
 # _test_run = timing.TimedRun(_test_lang, _test_lang)
 # _test_run.time_run(_test_count, _test_size, nrep=_test_nrep)
 _this_platform = (platform._platform,
@@ -307,10 +308,12 @@ class TestTimedRunTemp(TimedRunTestBase):
         out['overwrite'] = True
         return out
 
+    @flaky.flaky
     def test_pyperf_func(self):
         r"""Test pyperf_func."""
         timing.pyperf_func(1, self.instance, self.count, self.size, 0)
 
+    @flaky.flaky
     def test_run_overwrite(self):
         r"""Test performing a run twice, the second time with ovewrite."""
         self.time_run()
