@@ -1,6 +1,12 @@
 import numpy as np
 from yggdrasil.tests import YggTestBase
-from yggdrasil import units
+from yggdrasil import units, tools
+
+
+class DummyUnits(int):
+    r"""Dummy class for testing that has units attribute."""
+
+    units = 'cm'
 
 
 class TestUnits(YggTestBase):
@@ -10,6 +16,7 @@ class TestUnits(YggTestBase):
         r"""Setup, create variables for testing."""
         self._vars_nounits = [1.0, np.zeros(5), int(1)]
         self._vars_units = [units.add_units(v, 'cm') for v in self._vars_nounits]
+        self._vars_nounits.append(DummyUnits(1))
         super(TestUnits, self).setup(*args, **kwargs)
 
     def test_has_units(self):
@@ -19,6 +26,10 @@ class TestUnits(YggTestBase):
         for v in self._vars_units:
             assert(units.has_units(v))
 
+    def test_invalid_unit(self):
+        r"""Test error when an invalid unit is added."""
+        self.assert_raises(ValueError, units.add_units, 1.0, 'invalid')
+
     def test_get_data(self):
         r"""Test get_data."""
         for v in self._vars_nounits:
@@ -69,6 +80,8 @@ class TestUnits(YggTestBase):
         for v in self._vars_units:
             units.convert_to(v, 'm')
             self.assert_raises(ValueError, units.convert_to, v, 's')
+        x = units.add_units(int(1), 'umol')
+        units.convert_to(x, 'mol')
 
     def test_are_compatible(self):
         r"""Test are_compatible."""
@@ -83,6 +96,14 @@ class TestUnits(YggTestBase):
         r"""Test convert_R_unit_string."""
         pairs = [('g', 'g'), ('g2', '(g**2)'),
                  ('g2 km s-2', '(g**2)*km*(s**-2)'),
-                 ('h', 'hr')]
+                 ('degC d', 'degC*d'),
+                 (tools.bytes2str(b'\xc2\xb0C d'),
+                  tools.bytes2str(b'\xc2\xb0C*d')),
+                 ('h', 'hr'),
+                 ('hrs/kg', 'hr/kg'),
+                 ('', ''),
+                 ('cm**(-2)', '(cm**-2)')]
         for x, y in pairs:
             self.assert_equal(units.convert_R_unit_string(x), y)
+            self.assert_equal(units.convert_R_unit_string(y), y)
+            units.add_units(1.0, x)
